

function SplitPathName(pathName)
	return string.match(pathName, "(.-)([^\\/]-%.?([^%.\\/]*))$")
end

function GetScriptName() 
    return debug.getinfo(2, "S").source:sub(2) 
end 

local logging = require"logging"
require"logging.file"

local d,strScriptName,e = SplitPathName(GetScriptName())
local strResultsPath = string.format("%s_results", d)
local strLogFileName = string.format("%s/test.log", strResultsPath)--, strTestSubName)
local logger = logging.file(strLogFileName)
logger:info("running script %s", GetScriptName())

function RegisterClasses()
	return "CMapCoreLuaWrapper", "C3DMapLuaWrapper", "CSettingsLuaWrapper", "CWnd"--, "CEngine", "CRenderer"
end

local bExit = false
function OnAppSetWndMessage()
	if bExit then
		bExit = false
		return EWndMessage.WmQuit, 0, 0, 1
	else
		return 0
	end
end

function OnAppMapTimer(deltaTime)
	elapsedTime = deltaTime
	
	if MapCore.GetOverlayMode() ~= ENaviTypesOverlayMode.OmEdit then
		MapCore.SetOverlayMode(ENaviTypesOverlayMode.OmEdit)
	end
	
	co = co or coroutine.create(
		function()
			while true do
				for i = 1, #arrActions do
				
					local action = arrActions[i]
					
					if not action[1] or action[1]() then -- if allow function not defined or returns true
						action[2]() -- call action
						local actionTime = 0.0 
					
						-- wait for action to finish
						repeat
							coroutine.yield() -- suspend
							actionTime = actionTime + elapsedTime
						until action[3](actionTime, action[4])
					end
				end
				coroutine.yield()
			end
		end)
	
	coroutine.resume(co)
	--st = coroutine.status(co)
	--print(st)
end

function FlyTo()
	if not currPlace or currPlace >= #arrPlaces then currPlace = 1 else currPlace = currPlace + 1 end
	local place = arrPlaces[currPlace]
	Map3D.SetWantedPosition(place[1], place[2])
	str = string.format("flying to [%7d, %7d] %s", place[1], place[2], place[3])
	logger:info(str)
end

function RotateToNorth()
	Map3D.SetWantedRotation(0.0)
end

function SetDistance()
	wantedDistance = wantedDistance or 150.0 -- we start at 150.0 height
	Map3D.SetWantedDistance(wantedDistance)
	wantedDistance = wantedDistance * 1.5
end

function Set2DView()
	Settings.SetView(EDisplaySettingsMapView.MapView2D);
end

function Set3DView()
	Settings.SetView(EDisplaySettingsMapView.MapView3D);
end

function Elapsed(meanTime, timeToWait)
	--return meanTime > 0.1
	return meanTime > timeToWait
end

function AllowRotate()
	return MapCore.GetCurrentLod() == 2
end

function PassFinished()
	bExit = true;
	return false
end

arrActions = 
{
	-- action allowed	action fn		wait function	wait arg
	{ nil,				FlyTo, 			Elapsed, 		1.0 },
	{ nil,				SetDistance, 	Elapsed, 		1.0 },	
	{ nil,				RotateToNorth, 	Elapsed, 		2.0 },
	{ nil,				Set3DView, 		Elapsed, 		2.0 },

	{ PassFinished }
}

arrPlaces =
{
	{ 1637276, 4820920, "Wien"}
} 
