

function SplitPathName(pathName)
	return string.match(pathName, "(.-)([^\\/]-%.?([^%.\\/]*))$")
end

function GetScriptName() 
    return debug.getinfo(2, "S").source:sub(2) 
end 

local logging = require"logging"
require"logging.file"

local d,strScriptName,e = SplitPathName(GetScriptName())
local strResultsPath = string.format("%s_results", d)
local strLogFileName = string.format("%s/test.log", strResultsPath)--, strTestSubName)
local logger = logging.file(strLogFileName)
logger:info("running script %s", GetScriptName())

function RegisterClasses()
	return "CMapCoreLuaWrapper", "C3DMapLuaWrapper", "CSettingsLuaWrapper", "CWnd", "CEngine", "CRenderer"
end

local bTakeScreenshot = false
local nScreenshot = 0
function OnAppEngineSwap()
	-- take screenshot
	if bTakeScreenshot and Engine.bTakeScreenshot == 0 then
		Engine.bTakeScreenshot = 1
		bTakeScreenshot = false
		local strRendererName = Renderer.GetName()		
		local strNumber = string.format("%4d", nScreenshot)
		nScreenshot = nScreenshot + 1
		strNumber = string.gsub(strNumber, " ", "0")
		local strScreeshotFileName = string.format("%s/%s_%s_%s.bmp", strResultsPath, strScriptName, strNumber, strRendererName)
		logger:info("taking screenshot %s", strScreeshotFileName)
		return strScreeshotFileName, 1
	else
		return 0
	end
end
	
local bExit = false
function OnAppSetWndMessage()
	if bExit then
		bExit = false
		return EWndMessage.WmQuit, 0, 0, 1
	else
		return 0
	end
end

function OnAppMapTimer(deltaTime)
	elapsedTime = deltaTime
	
	if MapCore.GetOverlayMode() ~= ENaviTypesOverlayMode.OmEdit then
		MapCore.SetOverlayMode(ENaviTypesOverlayMode.OmEdit)
	end
	
	co = co or coroutine.create(
		function()
			while true do
				for i = 1, #arrActions do
				
					local action = arrActions[i]
					
					if not action[1] or action[1]() then -- if allow function not defined or returns true
						action[2]() -- call action
						local actionTime = 0.0 
					
						-- wait for action to finish
						repeat
							coroutine.yield() -- suspend
							actionTime = actionTime + elapsedTime
						until action[3](actionTime, action[4])
					end
				end
				coroutine.yield()
			end
		end)
	
	coroutine.resume(co)
	--st = coroutine.status(co)
	--print(st)
end

function FlyTo()
	if not currPlace or currPlace >= #arrPlaces then currPlace = 1 else currPlace = currPlace + 1 end
	local place = arrPlaces[currPlace]
	Map3D.SetWantedPosition(place[1], place[2])
	str = string.format("flying to [%7d, %7d] %s", place[1], place[2], place[3])
	logger:info(str)
end

function RotateToNorth()
	Map3D.SetWantedRotation(0.0)
end

function Rotate()
	if not wantedRotation or wantedRotation > 180.0 then wantedRotation = -180.0 end -- we start at -180.0 degrees (south)
	Map3D.SetWantedRotation(wantedRotation)
	wantedRotation = wantedRotation + 45.0
end

function SetDistance()
	wantedDistance = wantedDistance or 150.0 -- we start at 150.0 height
	Map3D.SetWantedDistance(wantedDistance)
	wantedDistance = wantedDistance * 1.5
end

function Set2DView()
	Settings.SetView(EDisplaySettingsMapView.MapView2D);
end

function Set3DView()
	Settings.SetView(EDisplaySettingsMapView.MapView3D);
end

function Elapsed(meanTime, timeToWait)
	--return meanTime > 0.1
	return meanTime > timeToWait
end

function AllowRotate()
	return MapCore.GetCurrentLod() == 2
end

function TakeScreenshot()
	bTakeScreenshot = true
	return false
end

local nPasses = 30
function PassFinished()
	nPasses = nPasses - 1
	if nPasses <= 0 then
		bExit = true;
	end
	
	return false
end

arrActions = 
{
	-- action allowed	action fn		wait function	wait arg
	{ nil,				SetDistance, 	Elapsed, 		3.0 },
	{ nil,				Set3DView, 		Elapsed, 		1.0 },
	{ nil,				RotateToNorth, 	Elapsed, 		2.0 },
	{ nil,				FlyTo, 			Elapsed, 		6.0 },
	{ TakeScreenshot },
	-- rotate 45 * 8 = 360 degrees
	{ AllowRotate,		Rotate, 		Elapsed, 		2.0 }, -- -180, -135
	{ nil,				Set2DView, 		Elapsed, 		3.0 },
	{ TakeScreenshot },
	{ AllowRotate, 		Rotate, 		Elapsed, 		2.0 }, -- -135,  -90
	{ nil,				Set3DView, 		Elapsed, 		1.0 },
	{ AllowRotate, 		Rotate, 		Elapsed, 		2.0 }, --  -90,  -45
	{ nil,				Set2DView, 		Elapsed, 		1.0 },
	{ AllowRotate, 		Rotate, 		Elapsed, 		2.0 }, --  -45,    0
	{ nil,				Set3DView, 		Elapsed, 		1.0 },
	{ AllowRotate,		Rotate, 		Elapsed, 		2.0 }, --    0,   45
	{ nil,				Set2DView, 		Elapsed, 		1.0 },
	{ AllowRotate, 		Rotate, 		Elapsed, 		2.0 }, --   45,   90
	{ nil,				Set3DView, 		Elapsed, 		1.0 },
	{ AllowRotate, 		Rotate, 		Elapsed, 		2.0 }, --   90,  135
	{ nil,				Set2DView, 		Elapsed, 		1.0 },
	{ AllowRotate, 		Rotate, 		Elapsed, 		2.0 }, --  135,  180
	{ PassFinished }
}

arrPlaces =
{
	{  235085, 4885689, "Paris"},
	{  744813, 4694799, "Bern"},
	{ 1341138, 5252339, "Berlin"},
	{ 1637276, 4820920, "Wien"},
	{ 3761767, 5575585, "Moskva"},
	{ -370033, 4041669, "Madrid"}
} 
